"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.blur = blur;
exports.collapseToAnchor = collapseToAnchor;
exports.collapseToEnd = collapseToEnd;
exports.collapseToFocus = collapseToFocus;
exports.collapseToStart = collapseToStart;
exports.collapseToEndOf = collapseToEndOf;
exports.collapseToEndOfNextBlock = collapseToEndOfNextBlock;
exports.collapseToEndOfNextText = collapseToEndOfNextText;
exports.collapseToEndOfPreviousBlock = collapseToEndOfPreviousBlock;
exports.collapseToEndOfPreviousText = collapseToEndOfPreviousText;
exports.collapseToStartOf = collapseToStartOf;
exports.collapseToStartOfNextBlock = collapseToStartOfNextBlock;
exports.collapseToStartOfNextText = collapseToStartOfNextText;
exports.collapseToStartOfPreviousBlock = collapseToStartOfPreviousBlock;
exports.collapseToStartOfPreviousText = collapseToStartOfPreviousText;
exports.extendBackward = extendBackward;
exports.extendForward = extendForward;
exports.extendToEndOf = extendToEndOf;
exports.extendToStartOf = extendToStartOf;
exports.focus = focus;
exports.moveBackward = moveBackward;
exports.moveForward = moveForward;
exports.moveTo = moveTo;
exports.moveToOffsets = moveToOffsets;
exports.moveToRangeOf = moveToRangeOf;
exports.moveStartOffset = moveStartOffset;
exports.moveEndOffset = moveEndOffset;
exports.unsetMarks = unsetMarks;
exports.snapshotSelection = snapshotSelection;
exports.unsetSelection = unsetSelection;

/**
 * Blur the selection.
 *
 * @param {Transform} transform
 */

function blur(transform) {
  var state = transform.state;
  var selection = state.selection;

  var sel = selection.blur();
  transform.setSelectionOperation(sel);
}

/**
 * Move the focus point to the anchor point.
 *
 * @param {Transform} transform
 */

function collapseToAnchor(transform) {
  var state = transform.state;
  var selection = state.selection;

  var sel = selection.collapseToAnchor();
  transform.setSelectionOperation(sel);
}

/**
 * Move the start point to the end point.
 *
 * @param {Transform} transform
 */

function collapseToEnd(transform) {
  var state = transform.state;
  var selection = state.selection;

  var sel = selection.collapseToEnd();
  transform.setSelectionOperation(sel);
}

/**
 * Move the anchor point to the focus point.
 *
 * @param {Transform} transform
 */

function collapseToFocus(transform) {
  var state = transform.state;
  var selection = state.selection;

  var sel = selection.collapseToFocus();
  transform.setSelectionOperation(sel);
}

/**
 * Move the end point to the start point.
 *
 * @param {Transform} transform
 */

function collapseToStart(transform) {
  var state = transform.state;
  var selection = state.selection;

  var sel = selection.collapseToStart();
  transform.setSelectionOperation(sel);
}

/**
 * Move to the end of a `node`.
 *
 * @param {Transform} transform
 * @param {Node} node
 */

function collapseToEndOf(transform, node) {
  var state = transform.state;
  var selection = state.selection;

  var sel = selection.collapseToEndOf(node);
  transform.setSelectionOperation(sel);
}

/**
 * Move the selection to the end of the next block.
 *
 * @param {Transform} tansform
 */

function collapseToEndOfNextBlock(transform) {
  var state = transform.state;
  var document = state.document,
      selection = state.selection;

  var blocks = document.getBlocksAtRange(selection);
  var last = blocks.last();
  var next = document.getNextBlock(last);
  if (!next) return;

  var sel = selection.collapseToEndOf(next);
  transform.setSelectionOperation(sel);
}

/**
 * Move the selection to the end of the next text.
 *
 * @param {Transform} tansform
 */

function collapseToEndOfNextText(transform) {
  var state = transform.state;
  var document = state.document,
      selection = state.selection;

  var texts = document.getTextsAtRange(selection);
  var last = texts.last();
  var next = document.getNextText(last);
  if (!next) return;

  var sel = selection.collapseToEndOf(next);
  transform.setSelectionOperation(sel);
}

/**
 * Move the selection to the end of the previous block.
 *
 * @param {Transform} tansform
 */

function collapseToEndOfPreviousBlock(transform) {
  var state = transform.state;
  var document = state.document,
      selection = state.selection;

  var blocks = document.getBlocksAtRange(selection);
  var first = blocks.first();
  var previous = document.getPreviousBlock(first);
  if (!previous) return;

  var sel = selection.collapseToEndOf(previous);
  transform.setSelectionOperation(sel);
}

/**
 * Move the selection to the end of the previous text.
 *
 * @param {Transform} tansform
 */

function collapseToEndOfPreviousText(transform) {
  var state = transform.state;
  var document = state.document,
      selection = state.selection;

  var texts = document.getTextsAtRange(selection);
  var first = texts.first();
  var previous = document.getPreviousText(first);
  if (!previous) return;

  var sel = selection.collapseToEndOf(previous);
  transform.setSelectionOperation(sel);
}

/**
 * Move to the start of a `node`.
 *
 * @param {Transform} transform
 * @param {Node} node
 */

function collapseToStartOf(transform, node) {
  var state = transform.state;
  var selection = state.selection;

  var sel = selection.collapseToStartOf(node);
  transform.setSelectionOperation(sel);
}

/**
 * Move the selection to the start of the next block.
 *
 * @param {Transform} tansform
 */

function collapseToStartOfNextBlock(transform) {
  var state = transform.state;
  var document = state.document,
      selection = state.selection;

  var blocks = document.getBlocksAtRange(selection);
  var last = blocks.last();
  var next = document.getNextBlock(last);
  if (!next) return;

  var sel = selection.collapseToStartOf(next);
  transform.setSelectionOperation(sel);
}

/**
 * Move the selection to the start of the next text.
 *
 * @param {Transform} tansform
 */

function collapseToStartOfNextText(transform) {
  var state = transform.state;
  var document = state.document,
      selection = state.selection;

  var texts = document.getTextsAtRange(selection);
  var last = texts.last();
  var next = document.getNextText(last);
  if (!next) return;

  var sel = selection.collapseToStartOf(next);
  transform.setSelectionOperation(sel);
}

/**
 * Move the selection to the start of the previous block.
 *
 * @param {Transform} tansform
 */

function collapseToStartOfPreviousBlock(transform) {
  var state = transform.state;
  var document = state.document,
      selection = state.selection;

  var blocks = document.getBlocksAtRange(selection);
  var first = blocks.first();
  var previous = document.getPreviousBlock(first);
  if (!previous) return;

  var sel = selection.collapseToStartOf(previous);
  transform.setSelectionOperation(sel);
}

/**
 * Move the selection to the start of the previous text.
 *
 * @param {Transform} tansform
 */

function collapseToStartOfPreviousText(transform) {
  var state = transform.state;
  var document = state.document,
      selection = state.selection;

  var texts = document.getTextsAtRange(selection);
  var first = texts.first();
  var previous = document.getPreviousText(first);
  if (!previous) return;

  var sel = selection.collapseToStartOf(previous);
  transform.setSelectionOperation(sel);
}

/**
 * Extend the focus point backward `n` characters.
 *
 * @param {Transform} transform
 * @param {Number} n (optional)
 */

function extendBackward(transform, n) {
  var state = transform.state;
  var document = state.document,
      selection = state.selection;

  var sel = selection.extendBackward(n).normalize(document);
  transform.setSelectionOperation(sel);
}

/**
 * Extend the focus point forward `n` characters.
 *
 * @param {Transform} transform
 * @param {Number} n (optional)
 */

function extendForward(transform, n) {
  var state = transform.state;
  var document = state.document,
      selection = state.selection;

  var sel = selection.extendForward(n).normalize(document);
  transform.setSelectionOperation(sel);
}

/**
 * Extend the focus point to the end of a `node`.
 *
 * @param {Transform} transform
 * @param {Node} node
 */

function extendToEndOf(transform, node) {
  var state = transform.state;
  var document = state.document,
      selection = state.selection;

  var sel = selection.extendToEndOf(node).normalize(document);
  transform.setSelectionOperation(sel);
}

/**
 * Extend the focus point to the start of a `node`.
 *
 * @param {Transform} transform
 * @param {Node} node
 */

function extendToStartOf(transform, node) {
  var state = transform.state;
  var document = state.document,
      selection = state.selection;

  var sel = selection.extendToStartOf(node).normalize(document);
  transform.setSelectionOperation(sel);
}

/**
 * Focus the selection.
 *
 * @param {Transform} transform
 */

function focus(transform) {
  var state = transform.state;
  var selection = state.selection;

  var sel = selection.focus();
  transform.setSelectionOperation(sel);
}

/**
 * Move the selection backward `n` characters.
 *
 * @param {Transform} transform
 * @param {Number} n (optional)
 */

function moveBackward(transform, n) {
  var state = transform.state;
  var document = state.document,
      selection = state.selection;

  var sel = selection.moveBackward(n).normalize(document);
  transform.setSelectionOperation(sel);
}

/**
 * Move the selection forward `n` characters.
 *
 * @param {Transform} transform
 * @param {Number} n (optional)
 */

function moveForward(transform, n) {
  var state = transform.state;
  var document = state.document,
      selection = state.selection;

  var sel = selection.moveForward(n).normalize(document);
  transform.setSelectionOperation(sel);
}

/**
 * Move the selection to a specific anchor and focus point.
 *
 * @param {Transform} transform
 * @param {Object} properties
 */

function moveTo(transform, properties) {
  transform.setSelectionOperation(properties);
}

/**
 * Move the selection to `anchor` and `focus` offsets.
 *
 * @param {Transform} transform
 * @param {Number} anchor
 * @param {Number} focus (optional)
 */

function moveToOffsets(transform, anchor, _focus) {
  var state = transform.state;
  var selection = state.selection;

  var sel = selection.moveToOffsets(anchor, _focus);
  transform.setSelectionOperation(sel);
}

/**
 * Move to the entire range of `start` and `end` nodes.
 *
 * @param {Transform} transform
 * @param {Node} start
 * @param {Node} end (optional)
 */

function moveToRangeOf(transform, start, end) {
  var state = transform.state;
  var document = state.document,
      selection = state.selection;

  var sel = selection.moveToRangeOf(start, end).normalize(document);
  transform.setSelectionOperation(sel);
}

/**
 * Move the start offset by `n`.
 *
 * @param {Transform} transform
 * @param {Number} n
 */

function moveStartOffset(transform, n) {
  var state = transform.state;
  var document = state.document,
      selection = state.selection;

  var sel = selection.moveStartOffset(n).normalize(document);
  transform.setSelectionOperation(sel);
}

/**
 * Move the end offset by `n`.
 *
 * @param {Transform} transform
 * @param {Number} n
 */

function moveEndOffset(transform, n) {
  var state = transform.state;
  var document = state.document,
      selection = state.selection;

  var sel = selection.moveEndOffset(n).normalize(document);
  transform.setSelectionOperation(sel);
}

/**
 * Unset the selection's marks.
 *
 * @param {Transform} transform
 */

function unsetMarks(transform) {
  transform.setSelectionOperation({ marks: null });
}

/**
 * Snapshot the current selection.
 *
 * @param {Transform} transform
 */

function snapshotSelection(transform) {
  var state = transform.state;
  var selection = state.selection;

  transform.setSelectionOperation(selection, { snapshot: true });
}

/**
 * Unset the selection, removing an association to a node.
 *
 * @param {Transform} transform
 */

function unsetSelection(transform) {
  transform.setSelectionOperation({
    anchorKey: null,
    anchorOffset: 0,
    focusKey: null,
    focusOffset: 0,
    isFocused: false,
    isBackward: false
  });
}